import unittest

# Define the test file names
test_files = [
    "testHistory.py",
    "testLayer.py",
    "testLRscheduler.py",
    "testModel.py",
    "testSimulationScheduler.py",
    "testSimulationScore.py",
    "testStorage.py",
    "unittests_autogenerated.py",
    "unittests_basic.py"
]

# Create a test loader
loader = unittest.TestLoader()

# Create an empty test suite
suite = unittest.TestSuite()

# Add tests from each test file to the test suite
for test_file in test_files:
    # Discover tests in the test file
    discovered_tests = loader.discover('./tests/', pattern=test_file)
    # Add discovered tests to the test suite
    suite.addTests(discovered_tests)

# Create a test runner that will output the results to the console
runner = unittest.TextTestRunner()

# Run the test suite
if __name__ == '__main__':
    runner.run(suite)
